const fs = require('fs')
const { promisify } = require('util')
const { logger } = require('@vuepress/shared-utils')
const { minify } = require('html-minifier')

const readFile = promisify(fs.readFile)
const writeFile = promisify(fs.writeFile)

module.exports = ({ filter = () => true, minifierOptions = {} }, ctx) => ({
  async generated(pagePaths) {
    if (!ctx.isProd) {
      return
    }
    logger.wait('Minify output htmls...')
    pagePaths = pagePaths.filter(filter)
    const defaultMinifierOptions = {
      collapseWhitespace: true,
      collapseBooleanAttributes: true,
      conservativeCollapse: true,
      collapseInlineTagWhitespace: true,
      html5: true,
      minifyCSS: true,
      minifyJS: true,
      minifyURLs: true,
      removeComments: true,
      sortAttributes: true,
      sortClassName: true,
    }
    for (const item of pagePaths) {
      const content = await readFile(item, 'utf8')
      const minifiedContent = minify(
        content,
        Object.assign(defaultMinifierOptions, minifierOptions)
      )
      await writeFile(item, minifiedContent, 'utf8')
    }
    logger.success('Minify output htmls finished.')
  },
})
